<?php
/*
Plugin Name: Fediverse Linker
Description: Converts a Fediverse username into a link.
Version: 1.4
Changes: Added option to show/hide prefix in links, adjusted settings layout, and added color picker for prefix icon.
Author: Mikka Luster
Author URI: https://mikka.is
*/

// Add settings to the Writing page
function fediverse_linker_settings() {
    
    add_settings_field(
        'fediverse_linker_prefix_position',
        'Prefix Position',
        'fediverse_linker_prefix_position_callback',
        'writing',
        'fediverse_linker_section'
    );

    add_settings_section(
        'fediverse_linker_section',
        'Fediverse Linker Settings',
        'fediverse_linker_section_callback',
        'writing'
    );

    add_settings_field(
        'fediverse_linker_prefix',
        'Fediverse Link Prefix',
        'fediverse_linker_prefix_callback',
        'writing',
        'fediverse_linker_section'
    );

    add_settings_field(
        'fediverse_linker_new_tab',
        'Open Links in New Tab',
        'fediverse_linker_new_tab_callback',
        'writing',
        'fediverse_linker_section'
    );

    add_settings_field(
        'fediverse_linker_prefix_color',
        'Prefix Color',
        'fediverse_linker_prefix_color_callback',
        'writing',
        'fediverse_linker_section'
    );

    register_setting('writing', 'fediverse_linker_prefix');
    register_setting('writing', 'fediverse_linker_new_tab');
    register_setting('writing', 'fediverse_linker_show_prefix');
    register_setting('writing', 'fediverse_linker_prefix_color');
    register_setting('writing', 'fediverse_linker_prefix_position');
}

function fediverse_linker_section_callback() {
    echo '<p>Configure settings for the Fediverse Linker plugin.</p>';
}

function fediverse_linker_prefix_callback() {
    $prefix = get_option('fediverse_linker_prefix', '⁂');
    echo "<input type='text' name='fediverse_linker_prefix' value='" . esc_attr($prefix) . "' style='width: 3em;' maxlength='3' />";
    
    $show_prefix = get_option('fediverse_linker_show_prefix', '1');
    echo " <label><input type='checkbox' name='fediverse_linker_show_prefix' value='1' " . checked(1, $show_prefix, false) . " /> Show prefix in links</label>";
    echo "<p style='margin-top: 5px;'><small>This symbol (⁂) is the Fediverse icon I like best. Learn more about this symbol at <a href='https://symbol.fediverse.info/' target='_blank'>symbol.fediverse.info</a>.</small></p>";
}

function fediverse_linker_new_tab_callback() {
    $new_tab = get_option('fediverse_linker_new_tab', '1');
    echo "<input type='checkbox' name='fediverse_linker_new_tab' value='1' " . checked(1, $new_tab, false) . " />";
}

function fediverse_linker_prefix_position_callback() {
    $position = get_option('fediverse_linker_prefix_position', 'before');
    echo "<select name='fediverse_linker_prefix_position'>
        <option value='before' " . selected($position, 'before', false) . ">Before link</option>
        <option value='after' " . selected($position, 'after', false) . ">After link</option>
    </select>";
}

function fediverse_linker_prefix_color_callback() {
    $color = get_option('fediverse_linker_prefix_color', '');
    echo "<input type='text' name='fediverse_linker_prefix_color' value='" . esc_attr($color) . "' class='fediverse-color-field' data-default-color='' />";
}

add_action('admin_init', 'fediverse_linker_settings');

// Enqueue color picker scripts and styles
function fediverse_linker_admin_enqueue($hook) {
    if ('options-writing.php' !== $hook) {
        return;
    }
    wp_enqueue_style('wp-color-picker');
    wp_enqueue_script('wp-color-picker');
    wp_add_inline_script('wp-color-picker', '
        jQuery(document).ready(function($){
            $(".fediverse-color-field").wpColorPicker();
        });
    ');
}
add_action('admin_enqueue_scripts', 'fediverse_linker_admin_enqueue');

// Update the fediverse_linker function to use the configurable prefix
function get_fediverse_prefix() {
    return get_option('fediverse_linker_prefix', '⁂');
}

function fediverse_linker($content) {
    $pattern = '/(?:fl:)?@(\w+)@([\w.-]+\.[a-z]{2,})/i';
    $prefix = get_fediverse_prefix();
    $new_tab = get_option('fediverse_linker_new_tab', '1');
    $show_prefix = get_option('fediverse_linker_show_prefix', '1');
    $prefix_color = get_option('fediverse_linker_prefix_color', '');
    $prefix_position = get_option('fediverse_linker_prefix_position', 'before');
    $target = $new_tab ? ' target="_blank"' : '';
    $prefix_style = $prefix_color ? " style='color: $prefix_color;'" : '';
    $prefix_html = $show_prefix ? "<span class='fediverse-prefix'$prefix_style>$prefix</span> " : '';
    
    $link_html = '<a class="fedilink"' . $target . ' href="https://$2/@$1">@$1</a>';
    $replacement = $prefix_position === 'before' ? $prefix_html . $link_html : $link_html . ' ' . $prefix_html;
    
    return preg_replace($pattern, $replacement, $content);
}

// Add explanation above comment form
function fediverse_linker_explanation() {
    echo '<p class="fediverse-linker-info">Du kannst auf Fediverse-Profile verlinken, indem du @benutzername@server.tld in deinem Kommentar eingibst.</p>';
}

add_filter('the_content', 'fediverse_linker');
add_filter('comment_text', 'fediverse_linker');
add_action('comment_form_top', 'fediverse_linker_explanation');

// Add settings link on plugin page
function fediverse_linker_settings_link($links) {
    $settings_link = '<a href="' . admin_url('options-writing.php#fediverse_linker_section') . '">Settings</a>';
    array_unshift($links, $settings_link);
    return $links;
}

$plugin = plugin_basename(__FILE__);
add_filter("plugin_action_links_$plugin", 'fediverse_linker_settings_link');